@@ -195,62 +195,36 @@ apr_byte_t oidc_cache_mutex_destroy(server_rec *s, oidc_cache_mutex_t *m) {
 		m->mutex = NULL;
 	}
 
-	return rv;
+	return (rv == APR_SUCCESS);
 }
 
-#define OIDC_CACHE_CRYPTO_JSON_KEY "c"
-
 /*
  * AES GCM encrypt using the crypto passphrase as symmetric key
  */
-static apr_byte_t oidc_cache_crypto_encrypt(request_rec *r, const char *plaintext, const char *key,
-		char **result) {
-	apr_byte_t rv = FALSE;
-	json_t *json = NULL;
-
-	json = json_object();
-	json_object_set_new(json, OIDC_CACHE_CRYPTO_JSON_KEY, json_string(plaintext));
-
-	rv = oidc_util_jwt_create(r, (const char*) key, json, result, FALSE);
-
-	if (json)
-		json_decref(json);
-
-	return rv;
+static apr_byte_t oidc_cache_crypto_encrypt(request_rec *r,
+		const char *plaintext, const char *key, char **result) {
+	return oidc_util_jwt_create(r, key, plaintext, result);
 }
 
 /*
  * AES GCM decrypt using the crypto passphrase as symmetric key
  */
-static apr_byte_t oidc_cache_crypto_decrypt(request_rec *r, const char *cache_value,
-		const char *key, char **plaintext) {
-
-	apr_byte_t rv = FALSE;
-	json_t *json = NULL;
-
-	rv = oidc_util_jwt_verify(r, (const char*) key, cache_value, &json, FALSE);
-	if (rv == FALSE)
-		goto end;
-
-	rv = oidc_json_object_get_string(r->pool, json, OIDC_CACHE_CRYPTO_JSON_KEY, plaintext, NULL);
-
-end:
-
-	if (json)
-		json_decref(json);
-
-	return rv;
+static apr_byte_t oidc_cache_crypto_decrypt(request_rec *r,
+		const char *cache_value, const char *key, char **plaintext) {
+	return oidc_util_jwt_verify(r, key, cache_value, plaintext);
 }
 
 /*
  * hash a cache key and a crypto passphrase so the result is suitable as an randomized cache key
  */
-static char* oidc_cache_get_hashed_key(request_rec *r, const char *passphrase, const char *key) {
-	char *input = apr_psprintf(r->pool, "%s:%s", passphrase, key);
+static char* oidc_cache_get_hashed_key(request_rec *r, const char *passphrase,
+		const char *key) {
+	const char *input = apr_psprintf(r->pool, "%s:%s", passphrase, key);
 	char *output = NULL;
-	if (oidc_util_hash_string_and_base64url_encode(r, OIDC_JOSE_ALG_SHA256, input, &output)
-			== FALSE) {
-		oidc_error(r, "oidc_util_hash_string_and_base64url_encode returned an error");
+	if (oidc_util_hash_string_and_base64url_encode(r, OIDC_JOSE_ALG_SHA256,
+			input, &output) == FALSE) {
+		oidc_error(r,
+				"oidc_util_hash_string_and_base64url_encode returned an error");
 		return NULL;
 	}
 	return output;
@@ -267,6 +241,7 @@ apr_byte_t oidc_cache_get(request_rec *r, const char *section, const char *key,
 	int encrypted = oidc_cfg_cache_encrypt(r);
 	apr_byte_t rc = TRUE;
 	char *msg = NULL;
+	char *cache_value = NULL;
 
 	oidc_debug(r, "enter: %s (section=%s, decrypt=%d, type=%s)", key, section,
 			encrypted, cfg->cache->name);
@@ -276,7 +251,6 @@ apr_byte_t oidc_cache_get(request_rec *r, const char *section, const char *key,
 		key = oidc_cache_get_hashed_key(r, cfg->crypto_passphrase, key);
 
 	/* get the value from the cache */
-	const char *cache_value = NULL;
 	if (cfg->cache->get(r, section, key, &cache_value) == FALSE) {
 		rc = FALSE;
 		goto out;
@@ -292,6 +266,12 @@ apr_byte_t oidc_cache_get(request_rec *r, const char *section, const char *key,
 		goto out;
 	}
 
+	if (cfg->crypto_passphrase == NULL) {
+		oidc_error(r,
+				"could not decrypt cache entry because " OIDCCryptoPassphrase " is not set");
+		goto out;
+	}
+
 	rc = oidc_cache_crypto_decrypt(r, cache_value, cfg->crypto_passphrase, value);
 
 out:
@@ -301,7 +281,7 @@ out:
 	if (rc == TRUE)
 		if (*value != NULL)
 			oidc_debug(r, "cache hit: return %d bytes %s",
-					*value ? (int )strlen(*value) : 0, msg);
+					*value ? (int )_oidc_strlen(*value) : 0, msg);
 		else
 			oidc_debug(r, "cache miss %s", msg);
 	else
@@ -325,7 +305,7 @@ apr_byte_t oidc_cache_set(request_rec *r, const char *section, const char *key,
 
 	oidc_debug(r,
 			"enter: %s (section=%s, len=%d, encrypt=%d, ttl(s)=%" APR_TIME_T_FMT ", type=%s)",
-			key, section, value ? (int )strlen(value) : 0, encrypted,
+			key, section, value ? (int )_oidc_strlen(value) : 0, encrypted,
 					apr_time_sec(expiry - apr_time_now()), cfg->cache->name);
 
 	/* see if we need to encrypt */
@@ -336,7 +316,13 @@ apr_byte_t oidc_cache_set(request_rec *r, const char *section, const char *key,
 			goto out;
 
 		if (value != NULL) {
-			if (oidc_cache_crypto_encrypt(r, value, cfg->crypto_passphrase, &encoded) == FALSE)
+			if (cfg->crypto_passphrase == NULL) {
+				oidc_error(r,
+						"could not encrypt cache entry because " OIDCCryptoPassphrase " is not set");
+				goto out;
+			}
+			if (oidc_cache_crypto_encrypt(r, value, cfg->crypto_passphrase,
+					&encoded) == FALSE)
 				goto out;
 			value = encoded;
 		}
@@ -348,7 +334,7 @@ apr_byte_t oidc_cache_set(request_rec *r, const char *section, const char *key,
 out:
 	/* log the result */
 	msg = apr_psprintf(r->pool, "%d bytes in %s cache backend for %skey %s",
-			(value ? (int) strlen(value) : 0),
+			(value ? (int) _oidc_strlen(value) : 0),
 			(cfg->cache->name ? cfg->cache->name : ""),
 			(encrypted ? "encrypted " : ""), (key ? key : ""));
 	if (rc == TRUE)
