@@ -274,7 +274,7 @@ static redisReply* oidc_cache_redis_exec(request_rec *r, oidc_cache_cfg_redis_t
 	va_list ap;
 
 	/* try to execute a command at max 2 times while reconnecting */
-	for (i = 0; i < OIDC_REDIS_MAX_TRIES; i++) {
+	for (i = 1; i <= OIDC_REDIS_MAX_TRIES; i++) {
 
 		/* connect */
 		if (context->connect(r, context) != APR_SUCCESS)
@@ -291,8 +291,16 @@ static redisReply* oidc_cache_redis_exec(request_rec *r, oidc_cache_cfg_redis_t
 			break;
 
 		/* something went wrong, log it */
-		oidc_error(r, "Redis command (attempt=%d to %s:%d) failed, disconnecting: '%s' [%s]", i, context->host_str, context->port, errstr,
-				reply ? reply->str : "<n/a>");
+		if (i < OIDC_REDIS_MAX_TRIES)
+			oidc_warn(r,
+					"Redis command (attempt=%d/%d to %s:%d) failed, disconnecting: '%s' [%s]",
+					i, OIDC_REDIS_MAX_TRIES, context->host_str, context->port,
+					errstr, reply ? reply->str : "<n/a>");
+		else
+			oidc_error(r,
+					"Redis command (attempt=%d/%d to %s:%d) failed, disconnecting: '%s' [%s]",
+					i, OIDC_REDIS_MAX_TRIES, context->host_str, context->port,
+					errstr, reply ? reply->str : "<n/a>");
 
 		/* free the reply (if there is one allocated) */
 		oidc_cache_redis_reply_free(&reply);
@@ -307,10 +315,11 @@ static redisReply* oidc_cache_redis_exec(request_rec *r, oidc_cache_cfg_redis_t
 /*
  * get a name/value pair from Redis
  */
-apr_byte_t oidc_cache_redis_get(request_rec *r, const char *section, const char *key,
-		const char **value) {
+apr_byte_t oidc_cache_redis_get(request_rec *r, const char *section,
+		const char *key, char **value) {
 
-	oidc_cfg *cfg = ap_get_module_config(r->server->module_config, &auth_openidc_module);
+	oidc_cfg *cfg = ap_get_module_config(r->server->module_config,
+			&auth_openidc_module);
 	oidc_cache_cfg_redis_t *context = (oidc_cache_cfg_redis_t*) cfg->cache_cfg;
 	redisReply *reply = NULL;
 	apr_byte_t rv = FALSE;
@@ -320,8 +329,8 @@ apr_byte_t oidc_cache_redis_get(request_rec *r, const char *section, const char
 		return FALSE;
 
 	/* get */
-	reply =
-			oidc_cache_redis_exec(r, context, "GET %s", oidc_cache_redis_get_key(r->pool, section, key));
+	reply = oidc_cache_redis_exec(r, context, "GET %s",
+			oidc_cache_redis_get_key(r->pool, section, key));
 
 	if (reply == NULL)
 		goto end;
@@ -339,8 +348,10 @@ apr_byte_t oidc_cache_redis_get(request_rec *r, const char *section, const char
 	}
 
 	/* do a sanity check on the returned value */
-	if ((reply->str == NULL) || (reply->len != strlen(reply->str))) {
-		oidc_error(r, "redisCommand reply->len (%d) != strlen(reply->str): '%s'", (int )reply->len, reply->str);
+	if ((reply->str == NULL) || (reply->len != _oidc_strlen(reply->str))) {
+		oidc_error(r,
+				"redisCommand reply->len (%d) != _oidc_strlen(reply->str): '%s'",
+				(int )reply->len, reply->str);
 		goto end;
 	}
 
@@ -351,6 +362,7 @@ apr_byte_t oidc_cache_redis_get(request_rec *r, const char *section, const char
 	rv = TRUE;
 
 end:
+
 	/* free the reply object resources */
 	oidc_cache_redis_reply_free(&reply);
 
