@@ -151,10 +151,10 @@ static char *oidc_cache_shm_get_key(request_rec *r, const char *section,
 	char *section_key = apr_psprintf(r->pool, "%s:%s", section, key);
 
 	/* check that the passed in key is valid */
-	if (strlen(section_key) >= OIDC_CACHE_SHM_KEY_MAX) {
+	if (_oidc_strlen(section_key) >= OIDC_CACHE_SHM_KEY_MAX) {
 		oidc_error(r,
 				"could not construct cache key since key size is too large (%d >= %d) (%s)",
-				(int )strlen(section_key), OIDC_CACHE_SHM_KEY_MAX, section_key);
+				(int )_oidc_strlen(section_key), OIDC_CACHE_SHM_KEY_MAX, section_key);
 		return NULL;
 	}
 
@@ -165,11 +165,11 @@ static char *oidc_cache_shm_get_key(request_rec *r, const char *section,
  * get a value from the shared memory cache
  */
 static apr_byte_t oidc_cache_shm_get(request_rec *r, const char *section,
-		const char *key, const char **value) {
+		const char *key, char **value) {
 
 	oidc_cfg *cfg = ap_get_module_config(r->server->module_config,
 			&auth_openidc_module);
-	oidc_cache_cfg_shm_t *context = (oidc_cache_cfg_shm_t *) cfg->cache_cfg;
+	oidc_cache_cfg_shm_t *context = (oidc_cache_cfg_shm_t*) cfg->cache_cfg;
 
 	int i;
 	const char *section_key = oidc_cache_shm_get_key(r, section, key);
@@ -190,7 +190,7 @@ static apr_byte_t oidc_cache_shm_get(request_rec *r, const char *section,
 			i++, OIDC_CACHE_SHM_ADD_OFFSET(t, cfg->cache_shm_entry_size_max)) {
 		const char *tablekey = t->section_key;
 
-		if ((tablekey != NULL) && (apr_strnatcmp(tablekey, section_key) == 0)) {
+		if ((tablekey != NULL) && (_oidc_strcmp(tablekey, section_key) == 0)) {
 
 			/* found a match, check if it has expired */
 			if (t->expires > apr_time_now()) {
@@ -240,12 +240,12 @@ static apr_byte_t oidc_cache_shm_set(request_rec *r, const char *section,
 
 	/* check that the passed in value is valid */
 	if ((value != NULL)
-			&& (strlen(value)
+			&& (_oidc_strlen(value)
 					> (cfg->cache_shm_entry_size_max
 							- sizeof(oidc_cache_shm_entry_t)))) {
 		oidc_error(r,
 				"could not store value since value size is too large (%lu > %lu); consider increasing " OIDCCacheShmEntrySizeMax "",
-				(unsigned long )strlen(value),
+				(unsigned long )_oidc_strlen(value),
 				(unsigned long )(cfg->cache_shm_entry_size_max
 						- sizeof(oidc_cache_shm_entry_t)));
 		return FALSE;
@@ -276,7 +276,7 @@ static apr_byte_t oidc_cache_shm_set(request_rec *r, const char *section,
 		}
 
 		/* see if a value already exists for this key */
-		if (apr_strnatcmp(t->section_key, section_key) == 0) {
+		if (_oidc_strcmp(t->section_key, section_key) == 0) {
 			match = t;
 			break;
 		}
@@ -312,8 +312,8 @@ static apr_byte_t oidc_cache_shm_set(request_rec *r, const char *section,
 	if (value != NULL) {
 
 		/* fill out the entry with the provided data */
-		strcpy(t->section_key, section_key);
-		strcpy(t->value, value);
+		_oidc_strcpy(t->section_key, section_key);
+		_oidc_strcpy(t->value, value);
 		t->expires = expiry;
 		t->access = current_time;
 
@@ -347,7 +347,8 @@ static int oidc_cache_shm_destroy(server_rec *s) {
 	}
 
 	if ((context) && (context->mutex)) {
-		rv = oidc_cache_mutex_destroy(s, context->mutex);
+		if (oidc_cache_mutex_destroy(s, context->mutex) != TRUE)
+			rv = APR_EGENERAL;
 		context->mutex = NULL;
 	}
 
