@@ -44,16 +44,8 @@
 #ifndef MOD_AUTH_OPENIDC_H_
 #define MOD_AUTH_OPENIDC_H_
 
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#  undef PACKAGE_NAME
-#  undef PACKAGE_STRING
-#  undef PACKAGE_TARNAME
-#  undef PACKAGE_VERSION
-#  undef PACKAGE_BUGREPORT
-#endif
+#include "const.h"
 
-#include <stdint.h>
 #include <httpd.h>
 #include <http_core.h>
 #include <http_config.h>
@@ -75,6 +67,8 @@
 APLOG_USE_MODULE(auth_openidc);
 #endif
 
+#define HAVE_APACHE_24 MODULE_MAGIC_NUMBER_MAJOR >= 20100714
+
 #ifndef OIDC_DEBUG
 #define OIDC_DEBUG APLOG_DEBUG
 #endif
@@ -83,6 +77,18 @@ APLOG_USE_MODULE(auth_openidc);
 #define APLOG_TRACE1 APLOG_DEBUG
 #endif
 
+#ifndef apr_uintptr_t
+#define apr_uintptr_t apr_uint64_t
+#endif
+
+#ifndef APR_UINT32_MAX
+#define APR_UINT32_MAX UINT32_MAX
+#endif
+
+#ifndef APR_INT64_MAX
+#define APR_INT64_MAX INT64_MAX
+#endif
+
 #define oidc_log(r, level, fmt, ...) ap_log_rerror(APLOG_MARK, level, 0, r,"%s: %s", __FUNCTION__, apr_psprintf(r->pool, fmt, ##__VA_ARGS__))
 #define oidc_slog(s, level, fmt, ...) ap_log_error(APLOG_MARK, level, 0, s, "%s: %s", __FUNCTION__, apr_psprintf(s->process->pconf, fmt, ##__VA_ARGS__))
 //#define oidc_log(r, level, fmt, ...) fprintf(stderr, "# %s: %s\n", __FUNCTION__, apr_psprintf(r->pool, fmt, ##__VA_ARGS__))
@@ -106,11 +112,11 @@ APLOG_USE_MODULE(auth_openidc);
 #endif
 
 /* keys for storing info in the request state */
-#define OIDC_REQUEST_STATE_KEY_IDTOKEN "i"
-#define OIDC_REQUEST_STATE_KEY_CLAIMS  "c"
-#define OIDC_REQUEST_STATE_KEY_DISCOVERY  "d"
-#define OIDC_REQUEST_STATE_KEY_AUTHN  "a"
-#define OIDC_REQUEST_STATE_KEY_SAVE "s"
+#define OIDC_REQUEST_STATE_KEY_IDTOKEN            "i"
+#define OIDC_REQUEST_STATE_KEY_CLAIMS             "c"
+#define OIDC_REQUEST_STATE_KEY_DISCOVERY          "d"
+#define OIDC_REQUEST_STATE_KEY_AUTHN              "a"
+#define OIDC_REQUEST_STATE_KEY_SAVE               "s"
 
 /* parameter name of the callback URL in the discovery response */
 #define OIDC_DISC_CB_PARAM "oidc_callback"
@@ -148,11 +154,13 @@ APLOG_USE_MODULE(auth_openidc);
 #define OIDC_PASS_IDTOKEN_AS_SERIALIZED 4
 
 /* pass userinfo as individual claims in headers (default) */
-#define OIDC_PASS_USERINFO_AS_CLAIMS      1
+#define OIDC_PASS_USERINFO_AS_CLAIMS       1
 /* pass userinfo payload as JSON object in header */
-#define OIDC_PASS_USERINFO_AS_JSON_OBJECT 2
+#define OIDC_PASS_USERINFO_AS_JSON_OBJECT  2
 /* pass userinfo as a JWT in header (when returned as a JWT) */
-#define OIDC_PASS_USERINFO_AS_JWT         4
+#define OIDC_PASS_USERINFO_AS_JWT          3
+/* pass as re-signed JWT including id_token claims */
+#define OIDC_PASS_USERINFO_AS_SIGNED_JWT   4
 
 #define OIDC_PASS_APP_INFO_AS_BASE64URL 1
 #define OIDC_PASS_APP_INFO_AS_LATIN1    2
@@ -244,6 +252,9 @@ APLOG_USE_MODULE(auth_openidc);
 #define OIDC_COOKIE_SAMESITE_LAX(c, r) \
 		c->cookie_same_site ? OIDC_COOKIE_EXT_SAME_SITE_LAX : OIDC_COOKIE_EXT_SAME_SITE_NONE(c, r)
 
+#define OIDC_ERROR_ENVVAR "OIDC_ERROR"
+#define OIDC_ERROR_DESC_ENVVAR "OIDC_ERROR_DESC"
+
 /* https://tools.ietf.org/html/draft-ietf-tokbind-ttrp-01 */
 #define OIDC_TB_CFG_PROVIDED_ENV_VAR     "Sec-Provided-Token-Binding-ID"
 /* https://www.ietf.org/id/draft-ietf-oauth-mtls-12 */
@@ -278,9 +289,10 @@ extern oidc_proto_pkce_t oidc_pkce_s256;
 extern oidc_proto_pkce_t oidc_pkce_referred_tb;
 
 typedef struct oidc_jwks_uri_t {
-	const char *url;
+	char *uri;
 	int refresh_interval;
-	int ssl_validate_server;
+	char *signed_uri;
+	oidc_jwk_t *jwk;
 } oidc_jwks_uri_t;
 
 typedef struct oidc_provider_t {
@@ -295,7 +307,7 @@ typedef struct oidc_provider_t {
 	char *registration_endpoint_url;
 	char *check_session_iframe;
 	char *end_session_endpoint;
-	char *jwks_uri;
+	oidc_jwks_uri_t jwks_uri;
 	apr_array_header_t *verify_public_keys;
 	char *client_id;
 	char *client_secret;
@@ -314,16 +326,12 @@ typedef struct oidc_provider_t {
 	char *scope;
 	char *response_type;
 	char *response_mode;
-	int jwks_refresh_interval;
 	int idtoken_iat_slack;
 	char *auth_request_params;
 	int session_max_duration;
 	oidc_proto_pkce_t *pkce;
 	int userinfo_refresh_interval;
-
-	apr_array_header_t *client_signing_keys;
-	apr_array_header_t *client_encryption_keys;
-
+	apr_array_header_t *client_keys;
 	char *client_jwks_uri;
 	char *id_token_signed_response_alg;
 	char *id_token_encrypted_response_alg;
@@ -345,6 +353,13 @@ typedef struct oidc_remote_user_claim_t {
 	const char *replace;
 } oidc_remote_user_claim_t;
 
+typedef struct oidc_apr_expr_t {
+#if HAVE_APACHE_24
+	ap_expr_info_t *expr;
+#endif
+	char *str;
+} oidc_apr_expr_t;
+
 typedef struct oidc_oauth_t {
 	int ssl_validate_server;
 	char *client_id;
@@ -451,8 +466,6 @@ typedef struct oidc_cfg {
 	char *claim_delimiter;
 	char *claim_prefix;
 	oidc_remote_user_claim_t remote_user_claim;
-	int pass_idtoken_as;
-	int pass_userinfo_as;
 	int cookie_http_only;
 	int cookie_same_site;
 
@@ -463,8 +476,10 @@ typedef struct oidc_cfg {
 	int provider_metadata_refresh_interval;
 
 	apr_hash_t *info_hook_data;
+
 	apr_hash_t *black_listed_claims;
 	apr_hash_t *white_listed_claims;
+	oidc_apr_expr_t *filter_claims_expr;
 
 	apr_byte_t state_input_headers;
 	apr_hash_t *redirect_urls_allowed;
@@ -473,8 +488,9 @@ typedef struct oidc_cfg {
 	apr_byte_t x_forwarded_headers;
 } oidc_cfg;
 
+void oidc_pre_config_init();
 int oidc_check_user_id(request_rec *r);
-#if MODULE_MAGIC_NUMBER_MAJOR >= 20100714
+#if HAVE_APACHE_24
 authz_status oidc_authz_checker_claim(request_rec *r, const char *require_args, const void *parsed_require_args);
 #ifdef USE_LIBJQ
 authz_status oidc_authz_checker_claims_expr(request_rec *r, const char *require_args, const void *parsed_require_args);
@@ -595,17 +611,14 @@ apr_byte_t oidc_oauth_get_bearer_token(request_rec *r, const char **access_token
 #define OIDC_CLAIM_SID             "sid"
 #define OIDC_CLAIM_EVENTS          "events"
 
-#define OIDC_JWK_X5T       "x5t"
 #define OIDC_JWK_KEYS      "keys"
-#define OIDC_JWK_USE       "use"
-#define OIDC_JWK_SIG       "sig"
-#define OIDC_JWK_ENC       "enc"
 
 #define OIDC_HOOK_INFO_FORMAT_JSON         "json"
 #define OIDC_HOOK_INFO_FORMAT_HTML         "html"
 #define OIDC_HOOK_INFO_TIMESTAMP           "iat"
 #define OIDC_HOOK_INFO_ACCES_TOKEN         "access_token"
 #define OIDC_HOOK_INFO_ACCES_TOKEN_EXP     "access_token_expires"
+#define OIDC_HOOK_INFO_ID_TOKEN_HINT       "id_token_hint"
 #define OIDC_HOOK_INFO_ID_TOKEN            "id_token"
 #define OIDC_HOOK_INFO_USER_INFO           "userinfo"
 #define OIDC_HOOK_INFO_SESSION             "session"
@@ -655,6 +668,7 @@ apr_byte_t oidc_oauth_get_bearer_token(request_rec *r, const char **access_token
 #define OIDC_APP_INFO_ID_TOKEN_PAYLOAD  "id_token_payload"
 #define OIDC_APP_INFO_USERINFO_JSON     "userinfo_json"
 #define OIDC_APP_INFO_USERINFO_JWT      "userinfo_jwt"
+#define OIDC_APP_INFO_SIGNED_JWT        "signed_jwt"
 
 typedef json_t oidc_proto_state_t;
 
@@ -685,7 +699,7 @@ void oidc_proto_state_set_prompt(oidc_proto_state_t *proto_state, const char *pr
 void oidc_proto_state_set_pkce_state(oidc_proto_state_t *proto_state, const char *pkce_state);
 void oidc_proto_state_set_timestamp_now(oidc_proto_state_t *proto_state);
 
-apr_byte_t oidc_proto_token_endpoint_auth(request_rec *r, oidc_cfg *cfg, const char *token_endpoint_auth, const char *client_id, const char *client_secret, const apr_array_header_t *client_signing_keys, const char *audience, apr_table_t *params, const char *bearer_access_token, char **basic_auth_str, char **bearer_auth_str);
+apr_byte_t oidc_proto_token_endpoint_auth(request_rec *r, oidc_cfg *cfg, const char *token_endpoint_auth, const char *client_id, const char *client_secret, const apr_array_header_t *client_keys, const char *audience, apr_table_t *params, const char *bearer_access_token, char **basic_auth_str, char **bearer_auth_str);
 
 char *oidc_proto_peek_jwt_header(request_rec *r, const char *jwt, char **alg);
 int oidc_proto_authorization_request(request_rec *r, struct oidc_provider_t *provider, const char *login_hint, const char *redirect_uri, const char *state, oidc_proto_state_t *proto_state, const char *id_token_hint, const char *code_challenge, const char *auth_request_params, const char *path_scope);
@@ -700,7 +714,7 @@ int oidc_proto_javascript_implicit(request_rec *r, oidc_cfg *c);
 apr_array_header_t *oidc_proto_supported_flows(apr_pool_t *pool);
 apr_byte_t oidc_proto_flow_is_supported(apr_pool_t *pool, const char *flow);
 apr_byte_t oidc_proto_validate_authorization_response(request_rec *r, const char *response_type, const char *requested_response_mode, char **code, char **id_token, char **access_token, char **token_type, const char *used_response_mode);
-apr_byte_t oidc_proto_jwt_verify(request_rec *r, oidc_cfg *cfg, oidc_jwt_t *jwt, const oidc_jwks_uri_t *jwks_uri, apr_hash_t *symmetric_keys, const char *alg);
+apr_byte_t oidc_proto_jwt_verify(request_rec *r, oidc_cfg *cfg, oidc_jwt_t *jwt, const oidc_jwks_uri_t *jwks_uri, int ssl_validate_server, apr_hash_t *symmetric_keys, const char *alg);
 apr_byte_t oidc_proto_validate_jwt(request_rec *r, oidc_jwt_t *jwt, const char *iss, apr_byte_t exp_is_mandatory, apr_byte_t iat_is_mandatory, int iat_slack, int token_binding_policy);
 apr_byte_t oidc_proto_generate_nonce(request_rec *r, char **nonce, int len);
 apr_byte_t oidc_proto_validate_aud_and_azp(request_rec *r, oidc_cfg *cfg, oidc_provider_t *provider, oidc_jwt_payload_t *id_token_payload);
@@ -720,15 +734,15 @@ apr_byte_t oidc_proto_validate_nonce(request_rec *r, oidc_cfg *cfg, oidc_provide
 apr_byte_t oidc_validate_redirect_url(request_rec *r, oidc_cfg *c, const char *redirect_to_url, apr_byte_t restrict_to_host, char **err_str, char **err_desc);
 
 // oidc_authz.c
-typedef apr_byte_t (*oidc_authz_match_claim_fn_type)(request_rec *, const char * const, const json_t * const);
-apr_byte_t oidc_authz_match_claim(request_rec *r, const char * const attr_spec, const json_t * const claims);
+typedef apr_byte_t (*oidc_authz_match_claim_fn_type)(request_rec *, const char * const, json_t *);
+apr_byte_t oidc_authz_match_claim(request_rec *r, const char * const attr_spec, json_t *claims);
 #ifdef USE_LIBJQ
-apr_byte_t oidc_authz_match_claims_expr(request_rec *r, const char * const attr_spec, const json_t * const claims);
+apr_byte_t oidc_authz_match_claims_expr(request_rec *r, const char * const attr_spec, json_t *claims);
 #endif
-#if MODULE_MAGIC_NUMBER_MAJOR < 20100714
-int oidc_authz_worker22(request_rec *r, const json_t *const claims, const require_line *const reqs, int nelts);
+#if HAVE_APACHE_24
+authz_status oidc_authz_worker24(request_rec *r, json_t *claims, const char *require_args, const void *parsed_require_args, oidc_authz_match_claim_fn_type match_claim_fn);
 #else
-authz_status oidc_authz_worker24(request_rec *r, const json_t * const claims, const char *require_args, const void *parsed_require_args, oidc_authz_match_claim_fn_type match_claim_fn);
+int oidc_authz_worker22(request_rec *r, json_t *claims, const require_line *const reqs, int nelts);
 #endif
 int oidc_oauth_return_www_authenticate(request_rec *r, const char *error, const char *error_description);
 
@@ -753,6 +767,7 @@ int oidc_oauth_return_www_authenticate(request_rec *r, const char *error, const
 #define OIDCCookiePath                       "OIDCCookiePath"
 #define OIDCInfoHook                         "OIDCInfoHook"
 #define OIDCWhiteListedClaims                "OIDCWhiteListedClaims"
+#define OIDCCryptoPassphrase                 "OIDCCryptoPassphrase"
 
 void *oidc_create_server_config(apr_pool_t *pool, server_rec *svr);
 void *oidc_merge_server_config(apr_pool_t *pool, void *BASE, void *ADD);
@@ -777,8 +792,11 @@ int oidc_dir_cfg_unauth_action(request_rec *r);
 apr_byte_t oidc_dir_cfg_unauth_expr_is_set(request_rec *r);
 int oidc_dir_cfg_unautz_action(request_rec *r);
 char *oidc_dir_cfg_unauthz_arg(request_rec *r);
-char *oidc_dir_cfg_path_auth_request_params(request_rec *r);
-char *oidc_dir_cfg_path_scope(request_rec *r);
+const char *oidc_dir_cfg_path_auth_request_params(request_rec *r);
+apr_array_header_t *oidc_dir_cfg_pass_user_info_as(request_rec *r);
+int oidc_dir_cfg_pass_id_token_as(request_rec *r);
+const char *oidc_dir_cfg_userinfo_claims_expr(request_rec *r);
+const char *oidc_dir_cfg_path_scope(request_rec *r);
 oidc_valid_function_t oidc_cfg_get_valid_endpoint_auth_function(oidc_cfg *cfg);
 int oidc_cfg_cache_encrypt(request_rec *r);
 int oidc_cfg_session_cache_fallback_to_cookie(request_rec *r);
@@ -789,7 +807,8 @@ int oidc_cfg_dir_refresh_access_token_before_expiry(request_rec *r);
 int oidc_cfg_dir_logout_on_error_refresh(request_rec *r);
 char *oidc_cfg_dir_state_cookie_prefix(request_rec *r);
 int oidc_cfg_delete_oldest_state_cookies(oidc_cfg *cfg);
-void oidc_cfg_provider_init(oidc_provider_t *provider);
+oidc_provider_t* oidc_cfg_provider_create(apr_pool_t *pool);
+oidc_provider_t* oidc_cfg_provider_copy(apr_pool_t *pool, const oidc_provider_t *src);
 void oidc_config_check_x_forwarded(request_rec *r, const apr_byte_t x_forwarded_headers);
 
 // oidc_util.c
@@ -802,7 +821,7 @@ const char *oidc_get_redirect_uri(request_rec *r, oidc_cfg *c);
 const char *oidc_get_redirect_uri_iss(request_rec *r, oidc_cfg *c, oidc_provider_t *provider);
 char *oidc_url_encode(const request_rec *r, const char *str, const char *charsToEncode);
 char *oidc_normalize_header_name(const request_rec *r, const char *str);
-apr_byte_t oidc_util_request_is_secure(request_rec *r, oidc_cfg *c);
+apr_byte_t oidc_util_request_is_secure(request_rec *r, const oidc_cfg *c);
 void oidc_util_set_cookie(request_rec *r, const char *cookieName, const char *cookieValue, apr_time_t expires, const char *ext);
 char *oidc_util_get_cookie(request_rec *r, const char *cookieName);
 apr_byte_t oidc_util_http_get(request_rec *r, const char *url, const apr_table_t *params, const char *basic_auth, const char *bearer_token, int ssl_validate_server, char **response, int timeout, const char *outgoing_proxy, apr_array_header_t *pass_cookies, const char *ssl_cert, const char *ssl_key, const char *ssl_key_pwd);
@@ -826,13 +845,13 @@ apr_byte_t oidc_util_issuer_match(const char *a, const char *b);
 int oidc_util_html_send_error(request_rec *r, const char *html_template, const char *error, const char *description, int status_code);
 apr_byte_t oidc_util_json_array_has_value(request_rec *r, json_t *haystack, const char *needle);
 void oidc_util_set_app_info(request_rec *r, const char *s_key, const char *s_value, const char *claim_prefix, apr_byte_t as_header, apr_byte_t as_env_var, int pass_as);
-void oidc_util_set_app_infos(request_rec *r, const json_t *j_attrs, const char *claim_prefix, const char *claim_delimiter, apr_byte_t as_header, apr_byte_t as_env_var, int pass_as);
+void oidc_util_set_app_infos(request_rec *r, json_t *j_attrs, const char *claim_prefix, const char *claim_delimiter, apr_byte_t as_header, apr_byte_t as_env_var, int pass_as);
 apr_hash_t *oidc_util_spaced_string_to_hashtable(apr_pool_t *pool, const char *str);
 apr_byte_t oidc_util_spaced_string_equals(apr_pool_t *pool, const char *a, const char *b);
 apr_byte_t oidc_util_spaced_string_contains(apr_pool_t *pool, const char *str, const char *match);
 apr_byte_t oidc_json_object_get_string(apr_pool_t *pool, json_t *json, const char *name, char **value, const char *default_value);
-apr_byte_t oidc_json_object_get_int(apr_pool_t *pool, json_t *json, const char *name, int *value, const int default_value);
-apr_byte_t oidc_json_object_get_bool(apr_pool_t *pool, json_t *json, const char *name, int *value, const int default_value);
+apr_byte_t oidc_json_object_get_int(const json_t *json, const char *name, int *value, const int default_value);
+apr_byte_t oidc_json_object_get_bool(const json_t *json, const char *name, int *value, const int default_value);
 char *oidc_util_html_escape(apr_pool_t *pool, const char *input);
 char *oidc_util_javascript_escape(apr_pool_t *pool, const char *input);
 void oidc_util_table_add_query_encoded_params(apr_pool_t *pool, apr_table_t *table, const char *params);
@@ -843,8 +862,8 @@ apr_byte_t oidc_util_regexp_first_match(apr_pool_t *pool, const char *input, con
 apr_byte_t oidc_util_json_merge(request_rec *r, json_t *src, json_t *dst);
 int oidc_util_cookie_domain_valid(const char *hostname, char *cookie_domain);
 apr_byte_t oidc_util_hash_string_and_base64url_encode(request_rec *r, const char *openssl_hash_algo, const char *input, char **output);
-apr_byte_t oidc_util_jwt_create(request_rec *r, const char *secret, json_t *payload, char **compact_encoded_jwt, apr_byte_t strip_header);
-apr_byte_t oidc_util_jwt_verify(request_rec *r, const char *secret, const char *compact_encoded_jwt, json_t **result, apr_byte_t stripped_header);
+apr_byte_t oidc_util_jwt_create(request_rec *r, const char *secret, const char *s_payload, char **compact_encoded_jwt);
+apr_byte_t oidc_util_jwt_verify(request_rec *r, const char *secret, const char *compact_encoded_jwt, char **s_payload);
 char *oidc_util_get_chunked_cookie(request_rec *r, const char *cookieName, int cookie_chunk_size);
 void oidc_util_set_chunked_cookie(request_rec *r, const char *cookieName, const char *cookieValue, apr_time_t expires, int chunkSize, const char *ext);
 apr_byte_t oidc_util_create_symmetric_key(request_rec *r, const char *client_secret, unsigned int r_key_len, const char *hash_algo, apr_byte_t set_kid, oidc_jwk_t **jwk);
@@ -854,7 +873,11 @@ char *oidc_util_http_query_encoded_url(request_rec *r, const char *url, const ap
 char *oidc_util_get_full_path(apr_pool_t *pool, const char *abs_or_rel_filename);
 apr_byte_t oidc_enabled(request_rec *r);
 char *oidc_util_http_form_encoded_data(request_rec *r, const apr_table_t *params);
-char* oidc_util_strcasestr(const char *s1, const char *s2);
+const char* oidc_util_strcasestr(const char *s1, const char *s2);
+oidc_jwk_t* oidc_util_key_list_first(const apr_array_header_t *key_list, int kty, const char *use);
+const char* oidc_util_jq_filter(request_rec *r, const char *input, const char *filter);
+char *oidc_util_apr_expr_parse(cmd_parms *cmd, const char *str, oidc_apr_expr_t **expr, apr_byte_t result_is_str);
+const char *oidc_util_apr_expr_exec(request_rec *r, const oidc_apr_expr_t *expr, apr_byte_t result_is_str);
 
 /* HTTP header constants */
 #define OIDC_HTTP_HDR_COOKIE                            "Cookie"
@@ -916,7 +939,7 @@ apr_byte_t oidc_metadata_provider_parse(request_rec *r, oidc_cfg *cfg, json_t *j
 apr_byte_t oidc_metadata_provider_is_valid(request_rec *r, oidc_cfg *cfg, json_t *j_provider, const char *issuer);
 apr_byte_t oidc_metadata_list(request_rec *r, oidc_cfg *cfg, apr_array_header_t **arr);
 apr_byte_t oidc_metadata_get(request_rec *r, oidc_cfg *cfg, const char *selected, oidc_provider_t **provider, apr_byte_t allow_discovery);
-apr_byte_t oidc_metadata_jwks_get(request_rec *r, oidc_cfg *cfg, const oidc_jwks_uri_t *jwks_uri, json_t **j_jwks, apr_byte_t *refresh);
+apr_byte_t oidc_metadata_jwks_get(request_rec *r, oidc_cfg *cfg, const oidc_jwks_uri_t *jwks_uri, int ssl_validate_server, json_t **j_jwks, apr_byte_t *refresh);
 apr_byte_t oidc_oauth_metadata_provider_parse(request_rec *r, oidc_cfg *c, json_t *j_provider);
 
 // oidc_session.c
